import java.util.*;
import java.io.*;

/**
 * <p>Title: CountMEssage</p>
 * <p>Description: This class reads the contents of all log files using the
 * index information provided by the index log file, counts the number of
 * messages of various types, and writes the counting result to the console
 * </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: SCE, Carleton University</p>
 * @author Liu, Qi
 * @version 1.0
 */

public class CountMessage{
    /**
     * an iterator on the index log file, generate a sequence of log file names
     * that contain the messages
     */
    private Iterator it;

    /**
             * a hash table to hold messageType-numberOfMessage pairs, the key is the type
             * of the message, and the value is the number of messages of this particular
     * type.
     */
    private Hashtable table;
    private String path;

    public CountMessage(String indexLogFileName){
        File f = new File(indexLogFileName);
        try{
            String canonicalPath = f.getCanonicalPath();
            int separator = canonicalPath.lastIndexOf("\\");
            path = canonicalPath.substring(0, separator + 1);
            System.out.println("path = " + path);
        } catch(IOException ex){
            System.out.println(
                    "The input file does not exist! please check whether the input path is correct.");
            System.exit(1);
        }
        it = new LogFileIterator(indexLogFileName);
        table = new Hashtable();
    }

    public Iterator getLogFileIterator(){
        return it;
    }

    public Hashtable getTable(){
        return table;
    }

    public String getPath(){
            return path;
        }

    public static void main(String[] args){
        if((args.length != 1) || (args[0].equals("-h")) || (args[0].equals("-?"))){
            System.out.println("Please input the index log file name!");
            System.out.println(
                    "\tUsage: java CountMessage indexLogFileName.log");
            System.out.println(
                    "\tNOTE: You can input the indexLogFileName.log using absolute path or relative path.");
            System.out.println("\t1>Example for absolute path:");
            System.out.println("\t  c:\\\\market\\\\Original\\\\new\\\\same\\\\market.log");
            System.out.println();
            System.out.println("\t2>Example for relative path:");
            System.out.println("\t  .\\\\new\\\\same\\\\market.log");
            System.out.println("\t If relative path is used, it is relative to the directory where this class is located!");
            System.out.println("\t If this class resides in the same directory as the log files, input the log file name without path part.");
            System.out.println();
            System.exit(0);
        }
        //System.out.println("Input file: " + args[0]);

        CountMessage counter = new CountMessage(args[0]);
        Iterator it = counter.getLogFileIterator();
        Hashtable table = counter.getTable();
        String line = null;
        String logFile = null;
        String messageType = null;
        StringTokenizer tokenizer = null;
        BufferedReader reader = null;
        int num = 0;

        try{
            while(it.hasNext()){
                num++; //keep track of how many log files have been counted
                //open a log file and read messages line by line
                logFile = counter.getPath() + (String)it.next();
                reader = new BufferedReader(new FileReader(logFile));
                while((line = reader.readLine()) != null){
                    tokenizer = new StringTokenizer(line.trim(), "/");
                    tokenizer.nextToken(); //0
                    tokenizer.nextToken(); //L
                    messageType = tokenizer.nextToken().trim();
                    if(table.containsKey(messageType)){
                        Integer oldValue = (Integer)table.get(messageType);
                        table.put(messageType,
                                  new Integer(oldValue.intValue() + 1));
                    } else{
                        table.put(messageType, new Integer(1));
                    }
                }
            }

            //get the results
            int total = 0;
            table = counter.getTable();
            System.out.println("There are " + num + " log files counted.");
            Enumeration en = table.keys();
            while(en.hasMoreElements()){
                String type = (String)en.nextElement();
                Integer number = (Integer)table.get(type);
                total += number.intValue();
                System.out.println("#" + type + " = " + number);
            }
            System.out.println("Total message: " + total);
        } catch(IOException ex){
            System.out.println("Message Counting IOException!");
            ex.printStackTrace();
        } finally{
            if(reader != null){
                try{
                    reader.close();
                } catch(IOException ex){
                    System.out.println("Cannot close log file!");
                    ex.printStackTrace();
                }
            }
        }
    }

}